@@ -24,6 +24,7 @@ See: http://docs.translatehouse.org/projects/translate-toolkit/en/latest/command
 for examples and usage instructions.
 """
 
+import six
 import warnings
 
 from translate.convert import accesskey, convert
@@ -66,7 +67,7 @@ def applytranslation(key, propunit, inunit, mixedkeys):
     return value
 
 
-class reprop:
+class reprop(object):
 
     def __init__(self, templatefile, inputstore, personality, encoding=None,
                  remove_untranslated=False):
@@ -95,7 +96,6 @@ class reprop:
             outputlines.append(outputstr)
         return u"".join(outputlines).encode(self.encoding)
 
-
     def _handle_accesskeys(self, inunit, currkey):
         value = inunit.target
         if self.personality.name == "mozilla":
@@ -166,15 +166,16 @@ class reprop:
                 delimiter = prespace + delimiter_char + postspace
             if key in self.inputstore.locationindex:
                 unit = self.inputstore.locationindex[key]
-                if not unit.istranslated() and bool(unit.source) and self.remove_untranslated:
+                if unit is None or not unit.istranslated() and bool(unit.source) and self.remove_untranslated:
                     returnline = u""
+                    self.inecho = False
                 else:
                     if unit.isfuzzy() and not self.includefuzzy or len(unit.target) == 0:
                         value = unit.source
                     else:
                         value = self._handle_accesskeys(unit, key)
                     self.inecho = False
-                    assert isinstance(value, unicode)
+                    assert isinstance(value, six.text_type)
                     returnline = "%(key)s%(del)s%(value)s%(term)s%(eol)s" % {
                         "key": "%s%s%s" % (self.personality.key_wrap_char,
                                            key,
@@ -189,7 +190,7 @@ class reprop:
             else:
                 self.inecho = True
                 returnline = line + eol
-        assert isinstance(returnline, unicode)
+        assert isinstance(returnline, six.text_type)
         return returnline
 
 
@@ -243,25 +244,24 @@ def main(argv=None):
     # handle command line options
     parser = convert.ConvertOptionParser(formats, usetemplates=True,
                                          description=__doc__)
-    parser.add_option("", "--personality", dest="personality",
-            default=properties.default_dialect, type="choice",
-            choices=properties.dialects.keys(),
-            help="override the input file format: %s (for .properties files, default: %s)" %
-                 (", ".join(properties.dialects.iterkeys()),
-                  properties.default_dialect),
-            metavar="TYPE")
-    parser.add_option("", "--encoding", dest="encoding", default=None,
-            help="override the encoding set by the personality",
-            metavar="ENCODING")
-    parser.add_option("", "--removeuntranslated", dest="remove_untranslated",
-            default=False, action="store_true",
-            help="remove key value from output if it is untranslated")
+    parser.add_option(
+        "", "--personality", dest="personality",
+        default=properties.default_dialect, type="choice",
+        choices=list(properties.dialects.keys()),
+        help="override the input file format: %s (for .properties files, default: %s)" % (
+            ", ".join(six.iterkeys(properties.dialects)), properties.default_dialect),
+        metavar="TYPE")
+    parser.add_option(
+        "", "--encoding", dest="encoding", default=None,
+        help="override the encoding set by the personality",
+        metavar="ENCODING")
     parser.add_threshold_option()
     parser.add_fuzzy_option()
+    parser.add_remove_untranslated_option()
     parser.passthrough.append("personality")
     parser.passthrough.append("encoding")
-    parser.passthrough.append("remove_untranslated")
     parser.run(argv)
 
+
 if __name__ == '__main__':
     main()
