@@ -28,7 +28,7 @@ from translate.convert import convert
 from translate.storage import factory
 
 
-class resub:
+class resub(object):
 
     def __init__(self, templatefile, inputstore):
         from translate.storage import subtitles
@@ -49,7 +49,7 @@ class resub:
                         unit.target = inputunit.target
                 else:
                     unit.target = unit.source
-        return str(self.templatestore)
+        return bytes(self.templatestore)
 
 
 def convertsub(inputfile, outputfile, templatefile, includefuzzy=False,
@@ -71,10 +71,10 @@ def convertsub(inputfile, outputfile, templatefile, includefuzzy=False,
 def main(argv=None):
     # handle command line options
     formats = {
-         ("po", "srt"): ("srt", convertsub),
-         ("po", "sub"): ("sub", convertsub),
-         ("po", "ssa"): ("ssa", convertsub),
-         ("po", "ass"): ("ass", convertsub),
+        ("po", "srt"): ("srt", convertsub),
+        ("po", "sub"): ("sub", convertsub),
+        ("po", "ssa"): ("ssa", convertsub),
+        ("po", "ass"): ("ass", convertsub),
     }
     parser = convert.ConvertOptionParser(formats, usetemplates=True,
                                          description=__doc__)
@@ -82,5 +82,6 @@ def main(argv=None):
     parser.add_fuzzy_option()
     parser.run(argv)
 
+
 if __name__ == '__main__':
     main()
