@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2008 Zuza Software Foundation
@@ -27,6 +26,8 @@ from translate.lang import common
 class fi(common.Common):
     """This class represents Finnish."""
 
-    validaccel = u"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ" + \
-                 u"1234567890" + \
-                 u"äöÄÖ"
+    validaccel = (u"abcdefghijklmnopqrstuvwxyz"
+                  u"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+                  u"1234567890"
+                  u"äö"
+                  u"ÄÖ")
