@@ -1,7 +1,6 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2010 Zuza Software Foundation
+# Copyright 2010, 2016 Zuza Software Foundation
 #
 # This file is part of Virtaal.
 #
@@ -29,4 +28,6 @@ from translate.lang import common
 class ta(common.Common):
     """This class represents Tamil."""
 
-    ignoretests = ["startcaps", "simplecaps"]
+    ignoretests = {
+        'all': ["simplecaps", "startcaps"],
+    }
