@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2013 Zuza Software Foundation
@@ -29,4 +28,6 @@ from translate.lang.zh import zh
 class zh_cn(zh):
     specialchars = u"←→↔×÷©…—‘’“”【】《》"
 
-    ignoretests = ["startcaps", "simplecaps", "acronyms"]
+    ignoretests = {
+        'all': ["acronyms", "simplecaps", "startcaps"],
+    }
