@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2004-2006,2008 Zuza Software Foundation
@@ -22,6 +21,7 @@
 """module for parsing html files for translation"""
 
 import re
+import six
 
 from six.moves import html_parser
 from six.moves.html_entities import name2codepoint
@@ -98,7 +98,7 @@ class htmlunit(base.TranslationUnit):
 
     def __init__(self, source=None):
         self.locations = []
-        self.setsource(source)
+        self.source = source
 
     def getsource(self):
         #TODO: Rethink how clever we should try to be with html entities.
@@ -170,7 +170,7 @@ class htmlfile(html_parser.HTMLParser, base.TranslationStore):
 
     def __init__(self, includeuntaggeddata=None, inputfile=None,
                  callback=None):
-        self.units = []
+        base.TranslationStore.__init__(self)
         self.filename = getattr(inputfile, 'name', None)
         self.currentblock = u""
         self.currentcomment = u""
@@ -195,7 +195,7 @@ class htmlfile(html_parser.HTMLParser, base.TranslationStore):
     def _simple_callback(self, string):
         return string
 
-    ENCODING_RE = re.compile('''<meta.*
+    ENCODING_RE = re.compile(b'''<meta.*
                                 content.*=.*?charset.*?=\s*?
                                 ([^\s]*)
                                 \s*?["']\s*?>
@@ -208,18 +208,14 @@ class htmlfile(html_parser.HTMLParser, base.TranslationStore):
         """
 
         result = self.ENCODING_RE.findall(htmlsrc)
-        encoding = None
         if result:
-            encoding = result[0]
-        return encoding
+            self.encoding = result[0].decode('ascii')
+        return self.encoding
 
     def do_encoding(self, htmlsrc):
         """Return the html text properly encoded based on a charset."""
-        charset = self.guess_encoding(htmlsrc)
-        if charset:
-            return htmlsrc.decode(charset)
-        else:
-            return htmlsrc.decode('utf-8')
+        self.guess_encoding(htmlsrc)
+        return htmlsrc.decode(self.encoding)
 
     def pi_escape(self, text):
         """Replaces all instances of process instruction with placeholders,
@@ -263,14 +259,14 @@ class htmlfile(html_parser.HTMLParser, base.TranslationStore):
         if self.has_translatable_content(text):
             unit = self.addsourceunit(text)
             unit.addlocation("%s+%s:%d" %
-                              (self.filename, ".".join(self.tag_path),
-                               self.currentpos))
+                             (self.filename, ".".join(self.tag_path),
+                              self.currentpos))
             unit.addnote(self.currentcomment)
 
     def has_translatable_content(self, text):
         """Check if the supplied HTML snippet has any content that needs to be
-        translated."""
-
+        translated.
+        """
         text = text.strip()
         result = re.findall('(?i).*(charset.*=.*)', text)
         if len(result) == 1:
@@ -295,10 +291,11 @@ class htmlfile(html_parser.HTMLParser, base.TranslationStore):
         if startend:
             selfclosing = u" /"
         if attrs != [] and attrs is not None:
-            return u"<%(tag)s %(attrs)s%(selfclosing)s>" % \
-                    {"tag": tag,
-                     "attrs": " ".join(['%s="%s"' % pair for pair in attrs]),
-                     "selfclosing": selfclosing}
+            return u"<%(tag)s %(attrs)s%(selfclosing)s>" % {
+                "tag": tag,
+                "attrs": " ".join(['%s="%s"' % pair for pair in attrs]),
+                "selfclosing": selfclosing
+            }
         else:
             return u"<%(tag)s%(selfclosing)s>" % {"tag": tag,
                                                   "selfclosing": selfclosing}
@@ -403,14 +400,14 @@ class htmlfile(html_parser.HTMLParser, base.TranslationStore):
 
     def handle_charref(self, name):
         """Handle entries in the form &#NNNN; e.g. &#8417;"""
-        self.handle_data(unichr(int(name)))
+        self.handle_data(six.unichr(int(name)))
 
     def handle_entityref(self, name):
         """Handle named entities of the form &aaaa; e.g. &rsquo;"""
         if name in ['gt', 'lt', 'amp']:
             self.handle_data("&%s;" % name)
         else:
-            self.handle_data(unichr(name2codepoint.get(name, u"&%s;" % name)))
+            self.handle_data(six.unichr(name2codepoint.get(name, u"&%s;" % name)))
 
     def handle_comment(self, data):
         # we can place comments above the msgid as translator comments!
