@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2006-2011 Zuza Software Foundation
@@ -20,10 +19,12 @@
 
 """Parent class for LISA standards (TMX, TBX, XLIFF)"""
 
+import six
+
 try:
     from lxml import etree
     from translate.misc.xml_helpers import (getText, getXMLlang, getXMLspace,
-                                            namespaced, setXMLlang, setXMLspace)
+                                            namespaced)
 except ImportError as e:
     raise ImportError("lxml is not installed. It might be possible to continue without support for XML formats.")
 
@@ -31,6 +32,7 @@ from translate.lang import data
 from translate.storage import base
 
 
+@six.python_2_unicode_compatible
 class LISAunit(base.TranslationUnit):
     """
     A single unit in the file.  Provisional work is done to make several
@@ -137,8 +139,9 @@ class LISAunit(base.TranslationUnit):
     target_dom = property(get_target_dom)
 
     def settarget(self, text, lang='xx', append=False):
-        """Sets the "target" string (second language), or alternatively
-        appends to the list"""
+        """Sets the "target" string (second language), or alternatively appends
+        to the list
+        """
         #XXX: we really need the language - can't really be optional, and we
         # need to propagate it
         if self._rich_target is not None:
@@ -146,10 +149,10 @@ class LISAunit(base.TranslationUnit):
         text = data.forceunicode(text)
         # Firstly deal with reinitialising to None or setting to identical
         # string
-        if self.gettarget() == text:
+        if self.target == text:
             return
-        languageNode = self.get_target_dom(None)
-        if not text is None:
+        languageNode = self.target_dom
+        if text is not None:
             if languageNode is None:
                 languageNode = self.createlanguageNode(lang, text, "target")
                 self.set_target_dom(languageNode, append)
@@ -157,7 +160,7 @@ class LISAunit(base.TranslationUnit):
                 if self.textNode:
                     terms = languageNode.iter(self.namespaced(self.textNode))
                     try:
-                        languageNode = terms.next()
+                        languageNode = next(terms)
                     except StopIteration as e:
                         pass
                 languageNode.text = text
@@ -166,7 +169,8 @@ class LISAunit(base.TranslationUnit):
 
     def gettarget(self, lang=None):
         """retrieves the "target" text (second entry), or the entry in the
-        specified language, if it exists"""
+        specified language, if it exists
+        """
         return self.getNodeText(self.get_target_dom(lang),
                                 getXMLspace(self.xmlelement,
                                             self._default_xml_space))
@@ -174,38 +178,10 @@ class LISAunit(base.TranslationUnit):
 
     def createlanguageNode(self, lang, text, purpose=None):
         """Returns a xml Element setup with given parameters to represent a
-        single language entry. Has to be overridden."""
+        single language entry. Has to be overridden.
+        """
         return None
 
-    def createPHnodes(self, parent, text):
-        """Create the text node in parent containing all the ph tags"""
-        matches = _getPhMatches(text)
-        if not matches:
-            parent.text = text
-            return
-
-        # Now we know there will definitely be some ph tags
-        start = matches[0].start()
-        pretext = text[:start]
-        if pretext:
-            parent.text = pretext
-        lasttag = parent
-        for i, m in enumerate(matches):
-            #pretext
-            pretext = text[start:m.start()]
-            # this will never happen with the first ph tag
-            if pretext:
-                lasttag.tail = pretext
-            #ph node
-            phnode = etree.SubElement(parent, self.namespaced("ph"))
-            phnode.set("id", str(i + 1))
-            phnode.text = m.group()
-            lasttag = phnode
-            start = m.end()
-        #post text
-        if text[start:]:
-            lasttag.tail = text[start:]
-
     def getlanguageNodes(self):
         """Returns a list of all nodes that contain per language information.
         """
@@ -235,17 +211,18 @@ class LISAunit(base.TranslationUnit):
             terms = languageNode.iterdescendants(self.namespaced(self.textNode))
             if terms is None:
                 return None
-            try:
-                return getText(terms.next(), xml_space)
-            except StopIteration:
-                # didn't have the structure we expected
-                return None
+            node = next(terms, None)
+            if node is not None:
+                return getText(node, xml_space)
+            # didn't have the structure we expected
+            return None
         else:
             return getText(languageNode, xml_space)
 
     def __str__(self):
+        # 'unicode' encoding keeps the unicode status of the output
         return etree.tostring(self.xmlelement, pretty_print=True,
-                              encoding='utf-8')
+                              encoding='unicode')
 
     def _set_property(self, name, value):
         self.xmlelement.attrib[name] = value
@@ -265,6 +242,7 @@ class LISAunit(base.TranslationUnit):
 
 class LISAfile(base.TranslationStore):
     """A class representing a file store for one of the LISA file formats."""
+
     UnitClass = LISAunit
     #The root node of the XML document:
     rootNode = ""
@@ -276,19 +254,18 @@ class LISAfile(base.TranslationStore):
     namespace = None
 
     def __init__(self, inputfile=None, sourcelanguage='en',
-                 targetlanguage=None, unitclass=None):
-        super(LISAfile, self).__init__(unitclass=unitclass)
+                 targetlanguage=None, **kwargs):
+        super(LISAfile, self).__init__(**kwargs)
         if inputfile is not None:
             self.parse(inputfile)
             assert self.document.getroot().tag == self.namespaced(self.rootNode)
         else:
             # We strip out newlines to ensure that spaces in the skeleton
             # doesn't interfere with the the pretty printing of lxml
-            self.parse(self.XMLskeleton.replace("\n", ""))
+            self.parse(self.XMLskeleton.replace("\n", "").encode('utf-8'))
             self.setsourcelanguage(sourcelanguage)
             self.settargetlanguage(targetlanguage)
             self.addheader()
-        self._encoding = "UTF-8"
 
     def addheader(self):
         """Method to be overridden to initialise headers, etc."""
@@ -307,8 +284,9 @@ class LISAfile(base.TranslationStore):
         return namespaced(self.namespace, name)
 
     def initbody(self):
-        """Initialises self.body so it never needs to be retrieved from the
-        XML again."""
+        """Initialises self.body so it never needs to be retrieved from the XML
+        again.
+        """
         self.namespace = self.document.getroot().nsmap.get(None, None)
         self.body = self.document.find('//%s' % self.namespaced(self.bodyNode))
 
@@ -324,10 +302,10 @@ class LISAfile(base.TranslationStore):
         if new:
             self.body.append(unit.xmlelement)
 
-    def __str__(self):
+    def serialize(self, out=None):
         """Converts to a string containing the file's XML"""
-        return etree.tostring(self.document, pretty_print=True,
-                              xml_declaration=True, encoding='utf-8')
+        self.document.write(out, pretty_print=True, xml_declaration=True,
+                            encoding='utf-8')
 
     def parse(self, xml):
         """Populates this object from the given xml string"""
@@ -339,7 +317,7 @@ class LISAfile(base.TranslationStore):
             xml = posrc
         parser = etree.XMLParser(strip_cdata=False)
         self.document = etree.fromstring(xml, parser).getroottree()
-        self._encoding = self.document.docinfo.encoding
+        self.encoding = self.document.docinfo.encoding
         self.initbody()
         assert self.document.getroot().tag == self.namespaced(self.rootNode)
         for entry in self.document.getroot().iterdescendants(self.namespaced(self.UnitClass.rootNode)):
