@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2004-2014 Zuza Software Foundation
@@ -44,11 +43,11 @@ Java
     pairs that are not translated.
 
     The following `.properties file description
-    <http://docs.oracle.com/javase/1.4.2/docs/api/java/util/Properties.html#load(java.io.InputStream)>`_
+    <http://docs.oracle.com/javase/7/docs/api/java/util/Properties.html#load(java.io.Reader)>`_
     gives a good references to the .properties specification.
 
     Properties file may also hold Java `MessageFormat
-    <http://docs.oracle.com/javase/1.4.2/docs/api/java/text/MessageFormat.html>`_
+    <http://docs.oracle.com/javase/7/docs/api/java/text/MessageFormat.html>`_
     messages.  No special handling is provided in this storage class for
     MessageFormat, but this may be implemented in future.
 
@@ -82,9 +81,14 @@ Comments supported:
 .. code-block:: properties
 
    # a comment
-   ! a comment
    // a comment (only at the beginning of a line)
-   /* a comment (not across multiple lines) */
+
+   # The following are # escaped to render in docs
+   # ! is standard but not widely supported
+   #! a comment
+   # /* is non-standard but used on some implementations
+   #/* a comment (not across multiple lines) */
+
 
 Name and Value pairs:
 
@@ -93,7 +97,8 @@ Name and Value pairs:
    # Delimiters
    key = value
    key : value
-   key value
+   # Whitespace delimiter
+   # key[sp]value
 
    # Space in key and around value
    \ key\ = \ value
@@ -105,7 +110,7 @@ Name and Value pairs:
 
    # Special cases
    # key with no value
-   key
+   //key (escaped; doesn't render in docs)
    # value no key (extractable in prop2po but not mergeable in po2prop)
    =value
 
@@ -115,10 +120,11 @@ Name and Value pairs:
 """
 
 import re
+import six
+from codecs import iterencode
 
 from translate.lang import data
 from translate.misc import quote
-from translate.misc.deprecation import deprecated
 from translate.storage import base
 
 
@@ -136,66 +142,6 @@ ending in :attr:`.labelsuffixes` into accelerator notation"""
 eol = "\n"
 
 
-def _find_delimiter(line, delimiters):
-    """Find the type and position of the delimiter in a property line.
-
-    Property files can be delimited by "=", ":" or whitespace (space for now).
-    We find the position of each delimiter, then find the one that appears
-    first.
-
-    :param line: A properties line
-    :type line: str
-    :param delimiters: valid delimiters
-    :type delimiters: list
-    :return: delimiter character and offset within *line*
-    :rtype: Tuple (delimiter char, Offset Integer)
-    """
-    delimiter_dict = {}
-    for delimiter in delimiters:
-        delimiter_dict[delimiter] = -1
-    delimiters = delimiter_dict
-    # Find the position of each delimiter type
-    for delimiter, pos in delimiters.iteritems():
-        prewhitespace = len(line) - len(line.lstrip())
-        pos = line.find(delimiter, prewhitespace)
-        while pos != -1:
-            if delimiters[delimiter] == -1 and line[pos-1] != u"\\":
-                delimiters[delimiter] = pos
-                break
-            pos = line.find(delimiter, pos + 1)
-    # Find the first delimiter
-    mindelimiter = None
-    minpos = -1
-    for delimiter, pos in delimiters.iteritems():
-        if pos == -1 or delimiter == u" ":
-            continue
-        if minpos == -1 or pos < minpos:
-            minpos = pos
-            mindelimiter = delimiter
-    if mindelimiter is None and delimiters.get(u" ", -1) != -1:
-        # Use space delimiter if we found nothing else
-        return (u" ", delimiters[" "])
-    if (mindelimiter is not None and
-        u" " in delimiters and
-        delimiters[u" "] < delimiters[mindelimiter]):
-        # If space delimiter occurs earlier than ":" or "=" then it is the
-        # delimiter only if there are non-whitespace characters between it and
-        # the other detected delimiter.
-        if len(line[delimiters[u" "]:delimiters[mindelimiter]].strip()) > 0:
-            return (u" ", delimiters[u" "])
-    return (mindelimiter, minpos)
-
-
-@deprecated("Use Dialect.find_delimiter instead")
-def find_delimeter(line):
-    """Misspelled function that is kept around in case someone relies on it.
-
-    .. deprecated:: 1.7.0
-       Use :func:`find_delimiter` instead
-    """
-    return _find_delimiter(line, DialectJava.delimiters)
-
-
 def is_line_continuation(line):
     """Determine whether *line* has a line continuation marker.
 
@@ -277,6 +223,7 @@ def _key_strip(key):
         newkey += key[len(newkey):len(newkey)+1]
     return newkey.lstrip()
 
+
 dialects = {}
 default_dialect = "java"
 
@@ -293,6 +240,7 @@ def get_dialect(dialect=default_dialect):
 
 class Dialect(object):
     """Settings for the various behaviours in key=value files."""
+
     name = None
     default_encoding = 'iso-8859-1'
     delimiters = None
@@ -308,12 +256,62 @@ class Dialect(object):
         # to override key methods
         if encoding != "utf-8":
             return quote.javapropertiesencode(string or u"")
-        return string or u""
+        return quote.java_utf8_properties_encode(string or u"")
 
     @classmethod
     def find_delimiter(cls, line):
-        """Find the delimiter"""
-        return _find_delimiter(line, cls.delimiters)
+        """Find the type and position of the delimiter in a property line.
+
+        Property files can be delimited by "=", ":" or whitespace (space for now).
+        We find the position of each delimiter, then find the one that appears
+        first.
+
+        :param line: A properties line
+        :type line: str
+        :param delimiters: valid delimiters
+        :type delimiters: list
+        :return: delimiter character and offset within *line*
+        :rtype: Tuple (delimiter char, Offset Integer)
+        """
+        delimiter_dict = {}
+        for delimiter in cls.delimiters:
+            delimiter_dict[delimiter] = -1
+        delimiters = delimiter_dict
+        # Find the position of each delimiter type
+        for delimiter, pos in six.iteritems(delimiters):
+            start_pos = len(line) - len(line.lstrip())  # Skip initial whitespace
+            if cls.key_wrap_char != '' and line[start_pos] == cls.key_wrap_char:
+                # Skip the key if it is delimited by some char
+                start_pos += 1
+                while (line[start_pos] != cls.key_wrap_char or line[start_pos-1] == u"\\"):
+                    start_pos += 1
+            pos = line.find(delimiter, start_pos)
+            while pos != -1:
+                if delimiters[delimiter] == -1 and line[pos-1] != u"\\":
+                    delimiters[delimiter] = pos
+                    break
+                pos = line.find(delimiter, pos + 1)
+        # Find the first delimiter
+        mindelimiter = None
+        minpos = -1
+        for delimiter, pos in six.iteritems(delimiters):
+            if pos == -1 or delimiter == u" ":
+                continue
+            if minpos == -1 or pos < minpos:
+                minpos = pos
+                mindelimiter = delimiter
+        if mindelimiter is None and delimiters.get(u" ", -1) != -1:
+            # Use space delimiter if we found nothing else
+            return (u" ", delimiters[" "])
+        if (mindelimiter is not None and
+            u" " in delimiters and
+            delimiters[u" "] < delimiters[mindelimiter]):
+            # If space delimiter occurs earlier than ":" or "=" then it is the
+            # delimiter only if there are non-whitespace characters between it and
+            # the other detected delimiter.
+            if len(line[delimiters[u" "]:delimiters[mindelimiter]].strip()) > 0:
+                return (u" ", delimiters[u" "])
+        return (mindelimiter, minpos)
 
     @classmethod
     def key_strip(cls, key):
@@ -341,7 +339,7 @@ class DialectJavaUtf8(DialectJava):
 
     @classmethod
     def encode(cls, string, encoding=None):
-        return quote.mozillapropertiesencode(string or u"")
+        return quote.java_utf8_properties_encode(string or u"")
 
 
 @register_dialect
@@ -358,7 +356,7 @@ class DialectMozilla(DialectJavaUtf8):
     @classmethod
     def encode(cls, string, encoding=None):
         """Encode the string"""
-        string = quote.mozillapropertiesencode(string or u"")
+        string = quote.java_utf8_properties_encode(string or u"")
         string = quote.mozillaescapemarginspaces(string or u"")
         return string
 
@@ -377,7 +375,7 @@ class DialectSkype(Dialect):
 
     @classmethod
     def encode(cls, string, encoding=None):
-        return quote.mozillapropertiesencode(string or u"")
+        return quote.java_utf8_properties_encode(string or u"")
 
 
 @register_dialect
@@ -423,9 +421,11 @@ class DialectStringsUtf8(DialectStrings):
     default_encoding = "utf-8"
 
 
+@six.python_2_unicode_compatible
 class propunit(base.TranslationUnit):
-    """An element of a properties file i.e. a name and value, and any
-    comments associated."""
+    """An element of a properties file i.e. a name and value, and any comments
+    associated.
+    """
 
     def __init__(self, source="", personality="java"):
         """Construct a blank propunit."""
@@ -477,15 +477,12 @@ class propunit(base.TranslationUnit):
             return self.personality.default_encoding
 
     def __str__(self):
-        """Convert to a string. Double check that unicode is handled
-        somehow here."""
-        source = self.getoutput()
-        assert isinstance(source, unicode)
-        return source.encode(self.encoding)
+        """Convert to a string."""
+        return self.getoutput()
 
     def getoutput(self):
-        """Convert the element back into formatted lines for a
-        .properties file"""
+        """Convert the element back into formatted lines for a .properties file
+        """
         notes = self.getnotes()
         if notes:
             notes += u"\n"
@@ -540,8 +537,8 @@ class propunit(base.TranslationUnit):
         self.comments = []
 
     def isblank(self):
-        """returns whether this is a blank element, containing only
-        comments."""
+        """returns whether this is a blank element, containing only comments.
+        """
         return not (self.name or self.value)
 
     def istranslatable(self):
@@ -556,11 +553,12 @@ class propunit(base.TranslationUnit):
 
 class propfile(base.TranslationStore):
     """this class represents a .properties file, made up of propunits"""
+
     UnitClass = propunit
 
     def __init__(self, inputfile=None, personality="java", encoding=None):
         """construct a propfile, optionally reading in from inputfile"""
-        super(propfile, self).__init__(unitclass=self.UnitClass)
+        super(propfile, self).__init__()
         self.personality = get_dialect(personality)
         self.encoding = encoding or self.personality.default_encoding
         self.filename = getattr(inputfile, 'name', '')
@@ -571,11 +569,11 @@ class propfile(base.TranslationStore):
             self.makeindex()
 
     def parse(self, propsrc):
-        """Read the source of a properties file in and include them
-        as units."""
-        text, encoding = self.detect_encoding(propsrc,
-            default_encodings=[self.personality.default_encoding, 'utf-8',
-                               'utf-16'])
+        """Read the source of a properties file in and include them as units.
+        """
+        text, encoding = self.detect_encoding(
+            propsrc, default_encodings=[self.personality.default_encoding,
+                                        'utf-8', 'utf-16'])
         if not text:
             raise IOError("Cannot detect encoding for %s." % (self.filename or
                                                               "given string"))
@@ -637,13 +635,12 @@ class propfile(base.TranslationStore):
         if inmultilinevalue or len(newunit.comments) > 0:
             self.addunit(newunit)
 
-    def __str__(self):
-        """Convert the units back to lines."""
-        lines = []
-        for unit in self.units:
-            lines.append(unit.getoutput())
-        uret = u"".join(lines)
-        return uret.encode(self.encoding)
+    def serialize(self, out):
+        """Write the units back to file."""
+        # Thanks to iterencode, a possible BOM is written only once
+        for chunk in iterencode((unit.getoutput() for unit in self.units), self.encoding):
+            out.write(chunk)
+
 
 class javafile(propfile):
     Name = "Java Properties"
