@@ -1,9 +1,7 @@
-#!/usr/bin/env python
-
 import os
 import subprocess
 import sys
-from cStringIO import StringIO
+from io import BytesIO
 
 from translate.storage import factory, mo, test_base
 
@@ -24,8 +22,9 @@ os.environ["PATH"] = os.pathsep.join([os.path.join(dir, "translate", "tools"),
 class TestMOUnit(test_base.TestTranslationUnit):
     UnitClass = mo.mounit
 
+
 posources = [
-r'''
+    r'''
 msgid ""
 msgstr ""
 "PO-Revision-Date: 2006-02-09 23:33+0200\n"
@@ -33,7 +32,7 @@ msgstr ""
 "Content-Type: text/plain; charset=UTF-8\n"
 "Content-Transfer-Encoding: 8-bit\n"
 ''',
-r'''
+    r'''
 msgid ""
 msgstr ""
 "PO-Revision-Date: 2006-02-09 23:33+0200\n"
@@ -44,25 +43,25 @@ msgstr ""
 msgid "plant"
 msgstr ""
 ''',
-# The following test is commented out, because the hash-size is different
-# compared to gettext, since we're not counting untranslated units.
-#r'''
-#msgid ""
-#msgstr ""
-#"PO-Revision-Date: 2006-02-09 23:33+0200\n"
-#"MIME-Version: 1.0\n"
-#"Content-Type: text/plain; charset=UTF-8\n"
-#"Content-Transfer-Encoding: 8-bit\n"
-#
-#msgid "plant"
-#msgstr ""
-#
-#msgid ""
-#"_: Noun\n"
-#"convert"
-#msgstr "bekeerling"
-#''',
-r'''
+    # The following test is commented out, because the hash-size is different
+    # compared to gettext, since we're not counting untranslated units.
+    #r'''
+    #msgid ""
+    #msgstr ""
+    #"PO-Revision-Date: 2006-02-09 23:33+0200\n"
+    #"MIME-Version: 1.0\n"
+    #"Content-Type: text/plain; charset=UTF-8\n"
+    #"Content-Transfer-Encoding: 8-bit\n"
+    #
+    #msgid "plant"
+    #msgstr ""
+    #
+    #msgid ""
+    #"_: Noun\n"
+    #"convert"
+    #msgstr "bekeerling"
+    #''',
+    r'''
 msgid ""
 msgstr ""
 "PO-Revision-Date: 2006-02-09 23:33+0200\n"
@@ -83,7 +82,7 @@ msgid ""
 "convert"
 msgstr "omskakel"
 ''',
-r'''
+    r'''
 msgid ""
 msgstr ""
 "PO-Revision-Date: 2006-02-09 23:33+0200\n"
@@ -107,7 +106,8 @@ msgstr "omskakel"
 msgid "tree"
 msgid_plural "trees"
 msgstr[0] ""
-''']
+''',
+]
 
 
 class TestMOFile(test_base.TestTranslationStore):
@@ -142,31 +142,28 @@ class TestMOFile(test_base.TestTranslationStore):
             print("PO source file")
             print(posource)
             PO_FILE, MO_MSGFMT, MO_POCOMPILE = self.get_mo_and_po()
+            posource = posource.encode('utf-8')
 
-            out_file = open(PO_FILE, 'w')
-            out_file.write(posource)
-            out_file.close()
+            with open(PO_FILE, 'wb') as out_file:
+                out_file.write(posource)
 
             subprocess.call(['msgfmt', PO_FILE, '-o', MO_MSGFMT])
             subprocess.call(['pocompile', '--errorlevel=traceback', PO_FILE, MO_POCOMPILE])
 
-            store = factory.getobject(StringIO(posource))
+            store = factory.getobject(BytesIO(posource))
             if store.isempty() and not os.path.exists(MO_POCOMPILE):
                 # pocompile doesn't create MO files for empty PO files, so we
                 # can skip the checks here.
                 continue
 
-            mo_msgfmt_f = open(MO_MSGFMT)
-            mo_pocompile_f = open(MO_POCOMPILE)
-
-            try:
+            with open(MO_MSGFMT, 'rb') as mo_msgfmt_f:
                 mo_msgfmt = mo_msgfmt_f.read()
-                print("msgfmt output:")
-                print(repr(mo_msgfmt))
+            print("msgfmt output:")
+            print(repr(mo_msgfmt))
+
+            with open(MO_POCOMPILE, 'rb') as mo_pocompile_f:
                 mo_pocompile = mo_pocompile_f.read()
-                print("pocompile output:")
-                print(repr(mo_pocompile))
-                assert mo_msgfmt == mo_pocompile
-            finally:
-                mo_msgfmt_f.close()
-                mo_pocompile_f.close()
+            print("pocompile output:")
+            print(repr(mo_pocompile))
+
+            assert mo_msgfmt == mo_pocompile
