@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2009, 2013 Zuza Software Foundation
@@ -24,6 +23,7 @@
 import logging
 import math
 import re
+import six
 import threading
 import time
 from sqlite3 import dbapi2
@@ -54,8 +54,8 @@ class TMDB(object):
         self.min_similarity = min_similarity
         self.max_length = max_length
 
-        if not isinstance(db_file, unicode):
-            db_file = unicode(db_file)  # don't know which encoding
+        if not isinstance(db_file, six.text_type):
+            db_file = six.text_type(db_file)  # don't know which encoding
         self.db_file = db_file
         # share connections to same database file between different instances
         if db_file not in self._tm_dbs:
@@ -74,7 +74,7 @@ class TMDB(object):
     def _get_connection(self, index):
         current_thread = threading.currentThread()
         if current_thread not in self._tm_db:
-            connection = dbapi2.connect(self.db_file.encode('utf-8'))
+            connection = dbapi2.connect(self.db_file.encode('utf-8') if six.PY2 else self.db_file)
             cursor = connection.cursor()
             self._tm_db[current_thread] = (connection, cursor)
         return self._tm_db[current_thread][index]
@@ -178,8 +178,9 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
             self.cursor.executescript(script)
 
     def preload_db(self):
-        """ugly hack to force caching of sqlite db file in memory for
-        improved performance"""
+        """ugly hack to force caching of sqlite db file in memory for improved
+        performance
+        """
         if self.fulltext:
             query = """SELECT COUNT(*) FROM sources s JOIN fulltext f ON s.sid = f.docid JOIN targets t on s.sid = t.sid"""
         else:
@@ -262,8 +263,9 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
         return count
 
     def add_list(self, units, source_lang, target_lang, commit=True):
-        """insert all units in list into the database, units are
-        represented as dictionaries"""
+        """insert all units in list into the database, units are represented as
+        dictionaries
+        """
         count = 0
         for unit in units:
             self.add_dict(unit, source_lang, target_lang, commit=False)
@@ -274,8 +276,8 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
 
     def translate_unit(self, unit_source, source_langs, target_langs):
         """return TM suggestions for unit_source"""
-        if isinstance(unit_source, str):
-            unit_source = unicode(unit_source, "utf-8")
+        if isinstance(unit_source, bytes):
+            unit_source = unit_source.decode("utf-8")
         if isinstance(source_langs, list):
             source_langs = [data.normalize_code(lang) for lang in source_langs]
             source_langs = ','.join(source_langs)
@@ -294,7 +296,7 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
         # split source into words, remove punctuation and special
         # chars, keep words that are at least 3 chars long
         unit_words = STRIP_REGEXP.sub(' ', unit_source).split()
-        unit_words = filter(lambda word: len(word) > 2, unit_words)
+        unit_words = list(filter(lambda word: len(word) > 2, unit_words))
 
         if self.fulltext and len(unit_words) > 3:
             logging.debug("fulltext matching")
@@ -325,7 +327,7 @@ DROP TRIGGER IF EXISTS sources_delete_trig;
                 })
         results.sort(key=lambda match: match['quality'], reverse=True)
         results = results[:self.max_candidates]
-        logging.debug("results: %s", unicode(results))
+        logging.debug("results: %s", six.text_type(results))
         return results
 
 
