@@ -25,6 +25,7 @@ for examples and usage instructions.
 """
 
 import os
+import six
 import sys
 
 from translate.misc import optrecurse
@@ -60,17 +61,10 @@ class ConflictOptionParser(optrecurse.RecursiveOptionParser):
         """sets the usage string - if usage not given, uses getusagestring for each option"""
         if usage is None:
             self.usage = "%prog " + " ".join([self.getusagestring(option) for option in self.option_list]) + \
-                    "\n  input directory is searched for PO files, PO files with name of conflicting string are output in output directory"
+                "\n  input directory is searched for PO files, PO files with name of conflicting string are output in output directory"
         else:
             super(ConflictOptionParser, self).set_usage(usage)
 
-    def run(self):
-        """parses the arguments, and runs recursiveprocess with the resulting options"""
-        (options, args) = self.parse_args()
-        options.inputformats = self.inputformats
-        options.outputoptions = self.outputoptions
-        self.recursiveprocess(options)
-
     def recursiveprocess(self, options):
         """recurse through directories and process files"""
         if self.isrecursive(options.input, 'input') and getattr(options, "allowrecursiveinput", True):
@@ -91,18 +85,16 @@ class ConflictOptionParser(optrecurse.RecursiveOptionParser):
             else:
                 inputfiles = [options.input]
         self.textmap = {}
-        self.initprogressbar(inputfiles, options)
+        progress_bar = optrecurse.ProgressBar(options.progress, inputfiles)
         for inputpath in inputfiles:
             fullinputpath = self.getfullinputpath(options, inputpath)
             try:
                 success = self.processfile(None, options, fullinputpath)
-            except Exception as error:
-                if isinstance(error, KeyboardInterrupt):
-                    raise
+            except Exception:
                 self.warning("Error processing: input %s" % (fullinputpath), options, sys.exc_info())
                 success = False
-            self.reportprogress(inputpath, success)
-        del self.progressbar
+            progress_bar.report_progress(inputpath, success)
+        del progress_bar
         self.buildconflictmap()
         self.outputconflicts(options)
 
@@ -145,7 +137,7 @@ class ConflictOptionParser(optrecurse.RecursiveOptionParser):
     def buildconflictmap(self):
         """work out which strings are conflicting"""
         self.conflictmap = {}
-        for source, translations in self.textmap.iteritems():
+        for source, translations in six.iteritems(self.textmap):
             source = self.flatten(source, " ")
             if len(source) <= 1:
                 continue
@@ -162,7 +154,7 @@ class ConflictOptionParser(optrecurse.RecursiveOptionParser):
         def str_len(x):
             return len(x)
 
-        for source, translations in self.conflictmap.iteritems():
+        for source, translations in six.iteritems(self.conflictmap):
             words = source.split()
             words.sort(key=str_len)
             source = words[-1]
@@ -172,26 +164,30 @@ class ConflictOptionParser(optrecurse.RecursiveOptionParser):
         for word in reducedmap:
             if word + "s" in reducedmap:
                 plurals[word] = word + "s"
-        for word, pluralword in plurals.iteritems():
+        for word, pluralword in six.iteritems(plurals):
             reducedmap[word].extend(reducedmap.pop(pluralword))
-        for source, translations in reducedmap.iteritems():
+        for source, translations in six.iteritems(reducedmap):
             flatsource = self.flatten(source, "-")
             fulloutputpath = os.path.join(options.output, flatsource + os.extsep + "po")
             conflictfile = po.pofile()
             for target, unit, filename in translations:
                 unit.othercomments.append("# (poconflicts) %s\n" % filename)
                 conflictfile.units.append(unit)
-            open(fulloutputpath, "w").write(str(conflictfile))
+            with open(fulloutputpath, "wb") as fh:
+                conflictfile.serialize(fh)
 
 
 def main():
     formats = {"po": ("po", None), None: ("po", None)}
     parser = ConflictOptionParser(formats)
-    parser.add_option("-I", "--ignore-case", dest="ignorecase",
+    parser.add_option(
+        "-I", "--ignore-case", dest="ignorecase",
         action="store_true", default=False, help="ignore case distinctions")
-    parser.add_option("-v", "--invert", dest="invert",
+    parser.add_option(
+        "-v", "--invert", dest="invert",
         action="store_true", default=False, help="invert the conflicts thus extracting conflicting destination words")
-    parser.add_option("", "--accelerator", dest="accelchars", default="",
+    parser.add_option(
+        "", "--accelerator", dest="accelchars", default="",
         metavar="ACCELERATORS", help="ignores the given accelerator characters when matching")
     parser.set_usage()
     parser.description = __doc__
